#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){	
    ofBackground(170);  //グレー背景を描画
    ofSetOrientation(OF_ORIENTATION_DEFAULT);   //画面の向きは標準
    
    camImg.load("button/camera.png");   //camera, library, save画像読み込み
    libImg.load("button/library.png");
    saveImg.load("button/save.png");

    //ボタンの位置を決定。rect.set(x, y, width, height)
    camRect.set(20, ofGetHeight() - camImg.getHeight() - 20, camImg.getWidth(), camImg.getHeight());
    libRect.set(100, ofGetHeight() - camImg.getHeight() - 20, camImg.getWidth(), camImg.getHeight());
    saveRect.set(ofGetWidth() - libImg.getWidth() - 20, ofGetHeight() - libImg.getHeight() - 20, libImg.getWidth(), libImg.getHeight());
    
    saveFlag = false;   //saveボタン用フラッグの初期値はfalse
}

//--------------------------------------------------------------
void ofApp::update(){
    
    if(camera.getImageUpdated()){
        
        unsigned char * pixels = camera.getPixels();    //カメラ画像データ
        
        int cameraW = camera.getWidth();    //撮影画像の幅、高さ
        int cameraH = camera.getHeight();
        
        displayPixels = new unsigned char [cameraW * cameraH * 4];  //変換後の画像ピクセルデータを格納する変数
        
        //1ピクセルずつ取り出して、r, g, bを抽出
        for (int y = 0; y < cameraH; y ++){
            for (int x = 0; x < cameraW; x ++){
                
                float r = pixels[y*cameraW*4 + x*4];    //赤を抽出
                float g = pixels[y*cameraW*4 + x*4 + 1];    //緑を抽出
                float b = pixels[y*cameraW*4 + x*4 + 2];    //青を抽出
                
                //effect 1//////////////////////
                //（色の入れ替え）
                
                r = g;   //赤　0 ~ 255
                g = b;   //緑　0 ~ 255
                b = r;   //青　0 ~ 255
                 
                
                //effect 2//////////////////////
                //（明るさ、コントラスト）
                /*
                 float brightness = 50;
                 float contrast = 2.5;
                 
                 r = contrast*(r - 128) + 128 + brightness;
                 g = contrast*(g - 128) + 128 + brightness;
                 b = contrast*(b - 128) + 128 + brightness;
                 */
                
                //effect 3//////////////////////
                //（赤っぽくする）
                /*
                 r = r;
                 g = g - 100;    //緑を引く
                 b = b - 100;    //青を引く
                 */
                
                //effect 4//////////////////////
                //（グレー）
                /*
                 r = (r + g + b) / 3;
                 g = (r + g + b) / 3;
                 b = (r + g + b) / 3;
                 */
                
                //effect 5//////////////////////
                //色の反転
                /*
                 r = 255 - r;
                 g = 255 - g;
                 b = 255 - b;
                */
                
                //effect 6//////////////////////
                //セピア
                /*
                 r = r/255*240;
                 g = g/255*200;
                 b = b/255*145;
                */
                
                //effect 7//////////////////////
                //2色
                
                /*
                 int brightness = (r + g + b)/3;
                 
                 if(brightness < 127){
                 
                    r = 32;     //赤
                    g = 225;     //緑
                    b = 0;     //青
                 
                 }else if(brightness >=127){
                 
                    r = 255;     //赤
                    g = 180;     //緑
                    b = 0;     //青
                 }
                 */
                
                //effect 8//////////////////////
                //3色
/*
                 int brightness = (r + g + b)/3;
                 
                 if(brightness < 85){
                 
                    r = 0;     //赤
                    g = 50;     //緑
                    b = 0;     //青
                 
                 }else if(brightness >=85 && brightness < 170){
                 
                    r = 0;     //赤
                    g = 150;     //緑
                    b = 255;     //青
                 
                 }else if(brightness >= 170){
                 
                    r = 255;     //赤
                    g = 100;     //緑
                    b = 0;     //青
                 }
                */
                
                //effect 9//////////////////////
                //収縮
                /*
                float D = 150;      //収縮率。値が少なくなるほど湾曲する
                float rad = 200;        //半径の大きさ
                 
                int x1 = rad*(x-cameraW/2)/sqrt(D*D+(x-cameraW/2)*(x-cameraW/2)+(y-cameraH/2)*(y-cameraH/2)) + cameraW/2;
                int y1 = rad*(y-cameraH/2)/sqrt(D*D+(x-cameraW/2)*(x-cameraW/2)+(y-cameraH/2)*(y-cameraH/2)) + cameraH/2;
                 
                if (x1 >= 0 && x1 < cameraW && y1 >= 0 && y1 < cameraH) {
                 
                    r = pixels[y1*cameraW*4 + x1*4];
                    g = pixels[y1*cameraW*4 + x1*4+1];
                    b = pixels[y1*cameraW*4 + x1*4+2];
                }
                */
                //effect 9終わり//////////////////////
                
                //変換後の値をdisplayPixelsで保存
                displayPixels[y*cameraW*4 + x*4] = MIN(MAX(r, 0), 255);   //赤
                displayPixels[y*cameraW*4 + x*4 + 1] = MIN(MAX(g, 0), 255);   //緑
                displayPixels[y*cameraW*4 + x*4 + 2] = MIN(MAX(b, 0), 255);   //青
                displayPixels[y*cameraW*4 + x*4 + 3] = 255;
            }
        }
        
        //effect 10//////////////////////
        //膨張
        /*
         float D = 250;   //膨張率。値が少なくなるほど膨張する
         float rad = 400;    //円の半径
         
         for (int y = 0; y < cameraH; y ++){
            for (int x = 0; x < cameraW; x ++){
         
                int x1 = rad*(x-cameraW/2)/sqrt(D*D+(x-cameraW/2)*(x-cameraW/2)+(y-cameraH/2)*(y-cameraH/2)) + cameraW/2;
                int y1 = rad*(y-cameraH/2)/sqrt(D*D+(x-cameraW/2)*(x-cameraW/2)+(y-cameraH/2)*(y-cameraH/2)) + cameraH/2;
         
                if (x1 >= 0 && x1 < cameraW && y1 >= 0 && y1 < cameraH) {
         
                    displayPixels[y1*cameraW*4 + x1*4] = pixels[y*cameraW*4 + x*4];
                    displayPixels[y1*cameraW*4 + x1*4+1] = pixels[y*cameraW*4 + x*4+1];
                    displayPixels[y1*cameraW*4 + x1*4+2] = pixels[y*cameraW*4 + x*4+2];
                    displayPixels[y1*cameraW*4 + x1*4+3] =  255;
                }
            }
         }
         
         for (int y = 0; y < cameraH; y ++){
            for (int x = 0; x < cameraW; x ++){
         
                float distance = ofDist(cameraW/2, cameraH/2, x, y);
                if (distance < rad-20) {
         
                    int r = displayPixels[y*cameraW*4 + x*4];
                    int g = displayPixels[y*cameraW*4 + x*4 + 1];
                    int b = displayPixels[y*cameraW*4 + x*4 + 2];
         
                    if((r + g + b)/3 == 0){
         
                        int pixCount= 0;
                        int totalR = 0;
                        int totalG = 0;
                        int totalB = 0;
         
                        for(int ny = y-2; ny <= y+2; ny ++){
                            for(int nx = x-2; nx <= x+2; nx ++){
         
                                if(displayPixels[ny*cameraW*4 + nx*4] + displayPixels[ny*cameraW*4 + nx*4 +1] + displayPixels[ny*cameraW*4 + nx*4 + 2] != 0){
         
                                    totalR += displayPixels[ny*cameraW*4 + nx*4];
                                    totalG += displayPixels[ny*cameraW*4 + nx*4 + 1];
                                    totalB += displayPixels[ny*cameraW*4 + nx*4 + 2];
         
                                    pixCount ++;
                                }
                            }
                        }
         
                        displayPixels[y*cameraW*4 + x*4] = totalR/pixCount;
                        displayPixels[y*cameraW*4 + x*4 + 1] = totalG/pixCount;
                        displayPixels[y*cameraW*4 + x*4 + 2] = totalB/pixCount;
                        displayPixels[y*cameraW*4 + x*4 + 3] = 255;
                    }
                }
            }
         }
         //effect 10終わり//////////////////////
         */
        
        //変更したピクセルを変数photoにコピー
        photo.setFromPixels(displayPixels, cameraW, cameraH, OF_IMAGE_COLOR_ALPHA);
        camera.close(); //カメラ終了
    }

}

//--------------------------------------------------------------
void ofApp::draw(){
    ofBackground(100, 100, 100);
    
    if(photo.isAllocated()){
        ofSetColor(255, 255, 255);
        //paper.draw(0, 0, ofGetWidth(), ofGetHeight());

        ofSetColor(255, 255, 255);
        photo.draw(0, 0);   //撮影した写真を描画
        
        //もしsaveボタンを押したら、画面キャプチャをとって、saveFlagをfalseに戻す
        if(saveFlag){
            ofxiOSScreenGrab(NULL); //フォトアルバムにスクリーンを保存
            saveFlag = false;
        }else{
            ofSetColor(255, 255, 255, 127); //半透明にする
            saveImg.draw(saveRect); //通常はsaveボタンを表示
        }
    }
    
    //saveボタンが押されていない場合にはカメラボタンとライブラリボタン（フォトアルバム用）を表示
    if(!saveFlag){
        ofSetColor(255, 255, 255, 127); //半透明にする
        camImg.draw(camRect);
        libImg.draw(libRect);
    }
}

//--------------------------------------------------------------
void ofApp::exit(){

}

//--------------------------------------------------------------
void ofApp::touchDown(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchMoved(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchUp(ofTouchEventArgs & touch){
    
    if(camRect.inside(touch.x, touch.y)){     //cameraボタンを押したら
        
        camera.setMaxDimension(MAX(1024, ofGetHeight()));   //高さは最大1024px
        camera.openCamera(1);    //カメラ起動
        
    }else if(libRect.inside(touch.x, touch.y)){ //ライブラリボタンを押したら
        
        camera.setMaxDimension(MAX(1024, ofGetHeight()));   //高さは最大1024px
        camera.openLibrary();   //library（フォトアルバム）を開く
        
    }else if(saveRect.inside(touch.x, touch.y)){    //saveボタンを押したら
        
        saveFlag = true;    //saveフラッグをtrueにする
    }
}

//--------------------------------------------------------------
void ofApp::touchDoubleTap(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchCancelled(ofTouchEventArgs & touch){
    
}

//--------------------------------------------------------------
void ofApp::lostFocus(){

}

//--------------------------------------------------------------
void ofApp::gotFocus(){

}

//--------------------------------------------------------------
void ofApp::gotMemoryWarning(){

}

//--------------------------------------------------------------
void ofApp::deviceOrientationChanged(int newOrientation){

}
